//
//  SARefreshHeader.m
//  fisher
//
//  Created by 余西安 on 16/1/29.
//  Copyright © 2016年 Sian. All rights reserved.
//

#define SAColor(rgb)  [UIColor colorWithRed:((rgb&0xFF0000)>>16)/255.0 green:((rgb&0xFF00)>>8)/255.0 blue:(rgb&0xFF)/255.0 alpha:1.0]
#import "SARefreshHeader.h"

@implementation SARefreshHeader

- (instancetype)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame]){
        self.stateLabel.font = [UIFont systemFontOfSize:11];
        self.stateLabel.textColor = [UIColor lightGrayColor];
        self.lastUpdatedTimeLabel.font = [UIFont systemFontOfSize:11];
        self.lastUpdatedTimeLabel.textColor = [UIColor lightGrayColor];
        [self setTitle:@"下拉刷新" forState:MJRefreshStateIdle];
        [self setTitle:@"释放刷新" forState:MJRefreshStatePulling];
        [self setTitle:@"更新中..." forState:MJRefreshStateRefreshing];
        [self setValue:@"0" forKeyPath:@"arrowView.alpha"];
    }
    return self;
}

// 重新布局子控件
- (void)placeSubviews
{
    [super placeSubviews];
    self.stateLabel.mj_y = MJRefreshHeaderHeight * 0.3;
    self.lastUpdatedTimeLabel.mj_y = MJRefreshHeaderHeight * 0.65;
    CGFloat refreshX = self.mj_w * 0.5 - 40;
    CGFloat refreshY = MJRefreshHeaderHeight * 0.55;
    self.refreshIcon.center = CGPointMake(refreshX, refreshY);
}

// RefreshState状态
- (void)setState:(MJRefreshState)state
{
    [super setState:state];
    // 隐藏原控件LoadingView(小菊花效果)
    [self setValue:@(YES) forKeyPath:@"loadingView.hidden"];
    // 不动的状态给出不同的底部图标，辅助动画效果
    switch (state) {
        case MJRefreshStateIdle:{       // 下拉刷新
            self.refreshIcon.image = [UIImage imageNamed:@"icon_public_refresh_normal.png"];
        }break;
        case MJRefreshStatePulling:{    // 释放刷新
            self.refreshIcon.image = [UIImage imageNamed:@"icon_public_refresh_highlight.png"];
        }break;
        case MJRefreshStateRefreshing:{ // 更新中...
            self.refreshIcon.image = [UIImage imageNamed:@"icon_public_refresh_highlight.png"];
        }break;
        default:break;
    }
}

// KVO监听ScrollView ContentOffset值
- (void)scrollViewContentOffsetDidChange:(NSDictionary *)change
{
    [super scrollViewContentOffsetDidChange:change];
    // 当下拉时执行相关绘图动画
    if (self.state == MJRefreshStateIdle){
        CGFloat distance = self.scrollView.contentOffset.y;
        [self refreshWithDistance:distance];
    }
}

// 下拉距离(根据该值来画圆的弧度大小)
- (void)refreshWithDistance:(CGFloat)distance
{
    // 下拉最大距离为常量MJRefreshHeaderHeight，strokeEnd取值0~1
    CGFloat strokeEnd = distance / MJRefreshHeaderHeight;
    if (strokeEnd < 0) strokeEnd *= -1;
    self->_shapeLayer.strokeEnd = strokeEnd;
    [self.shapeLayer setNeedsDisplay];
}

// 修复自动刷新时无效果的Bug
- (void)beginRefreshing
{
    [super beginRefreshing];
    self.state = MJRefreshStateRefreshing;
}

#pragma mark - 懒加载成员属性
- (UIImageView *)refreshIcon
{
    if (self->_refreshIcon == nil){
        UIImage *image = [UIImage imageNamed:@"icon_public_refresh_normal.png"];
        self->_refreshIcon = [[UIImageView alloc] initWithImage:image];
        [self addSubview:self->_refreshIcon];
    }
    return self->_refreshIcon;
}
// 动画图层
- (CAShapeLayer *)shapeLayer
{
    if (self->_shapeLayer == nil){
        // 创建形状图层，图层大小与覆盖的小图标大小一致
        self->_shapeLayer = [CAShapeLayer layer];
        self->_shapeLayer.frame = self.refreshIcon.bounds;
        //  从0度开始、线度1像素、边框蓝色、内填充无色
        self->_shapeLayer.strokeStart =  0;
        self->_shapeLayer.lineWidth = 1.0f;
        self->_shapeLayer.fillColor = [[UIColor clearColor] CGColor];
        self->_shapeLayer.strokeColor = [SAColor(0x007AFF) CGColor];
        // 动画轨迹通过贝塞尔曲线实现，慢慢调出与图标重叠的轨迹
        CGFloat width = self.refreshIcon.bounds.size.width - 4;
        CGRect rect = CGRectMake(2, 2, width, width);
        UIBezierPath *path = [UIBezierPath bezierPathWithOvalInRect:rect];
        self->_shapeLayer.path = path.CGPath;
        [self.refreshIcon.layer addSublayer:self->_shapeLayer];
        // 逆时钟旋转90度，使动画从最顶端开始
        [self->_shapeLayer setTransform:CATransform3DMakeRotation(M_PI * -0.5, 0, 0, 1)];
    }
    return self->_shapeLayer;
}
@end
